package cn.rankin.cmsweb.controller.product;

import cn.rankin.cmsweb.service.product.GroupService;
import cn.rankin.cmsweb.service.user.utils.MerchantServiceUtil;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.GroupDTO;
import cn.rankin.common.utils.dto.search.GroupSearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.vo.GroupVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Slf4j
@RestController
@RequestMapping(value = "/group")
public class GroupController {

    @Autowired
    private GroupService groupService;

    @Autowired
    private MerchantServiceUtil<GroupVo> merchantServiceUtil;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<GroupVo>> search(GroupSearchDTO searchDTO) {
        APIResult<Page<GroupVo>> pageAPIResult = groupService.search(BeanUtil.convertToMap(searchDTO));
        merchantServiceUtil.setPageMerchantInfo(pageAPIResult);
        return pageAPIResult;
    }

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<GroupVo> create(@Valid @RequestBody GroupDTO groupDTO) {
        APIResult<GroupVo> groupVoAPIResult = groupService.create(groupDTO);
        merchantServiceUtil.setMerchantInfo(groupVoAPIResult);
        return groupVoAPIResult;
    }

    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<GroupVo> update(@RequestBody GroupDTO groupDTO) {
        APIResult<GroupVo> groupVoAPIResult = groupService.update(groupDTO);
        merchantServiceUtil.setMerchantInfo(groupVoAPIResult);
        return groupVoAPIResult;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String id) {
        return groupService.delete(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<GroupVo> getGroup(@PathVariable("id") String id) {
        APIResult<GroupVo> groupVoAPIResult = groupService.getGroup(id);
        merchantServiceUtil.setMerchantInfo(groupVoAPIResult);
        return groupVoAPIResult;
    }
}
