package cn.rankin.cmsweb.service.product.proxy;

import cn.rankin.cmsweb.service.product.WareService;
import cn.rankin.cmsweb.service.resource.ResourceService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.WareDTO;
import cn.rankin.common.utils.dto.search.SearchDTO;
import cn.rankin.common.utils.util.BeanUtil;
import cn.rankin.data.api.product.vo.WareVo;
import cn.rankin.common.utils.vo.resource.ResourceVo;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class WareServiceProxy {

    @Autowired
    private WareService wareService;

    @Autowired
    private ResourceService resourceService;

    public APIResult<Page<WareVo>> search(SearchDTO searchDTO) {
        Map<String, Object> params = BeanUtil.convertToMap(searchDTO);
        APIResult<Page<WareVo>> result = wareService.search(params);
        if (result.getSuccess()) {
            List<WareVo> wareVoList = result.getData().getList();
            setWareUrl(wareVoList);
            log.info(JSON.toJSONString(wareVoList));
        }
        return result;
    }

    public APIResult<WareVo> create(WareDTO wareDTO) {
        APIResult<WareVo> result = wareService.create(wareDTO);
        if (result.getSuccess()) {
            WareVo wareVo = result.getData();
            setWareUrl(wareVo);
        }
        return result;
    }

    public APIResult<WareVo> update(WareDTO wareDTO) {
        APIResult<WareVo> result = wareService.update(wareDTO);
        if (result.getSuccess()) {
            WareVo wareVo = result.getData();
            setWareUrl(wareVo);
        }
        return result;
    }

    public APIResult delete(String id) {
        return wareService.delete(id);
    }

    public APIResult<WareVo> getWare(String id) {
        APIResult<WareVo> result = wareService.getWare(id);
        if (result.getSuccess()) {
            WareVo wareVo = result.getData();
            setWareUrl(wareVo);
        }
        return result;
    }

    public Map<String, ResourceVo> getResourceMap(List<String> idList) {
        Map<String, ResourceVo> resourceVoMap = new HashMap<>();
        if (CollectionUtils.isEmpty(idList)) {
            return resourceVoMap;
        }

        APIResult<Map<String, ResourceVo>> apiResult = resourceService.getBatch(idList);
        if (!apiResult.getSuccess()) {
            log.warn("get resource map error!");
            return resourceVoMap;
        }else if (apiResult.getData() == null) {
            log.warn("get resource map empty");
            return resourceVoMap;
        }

        return apiResult.getData();
    }

    public void setWareUrl(List<WareVo> wareVoList) {
        if (wareVoList == null) {
            return;
        }

        List<String> idList = new ArrayList<>();
        wareVoList.forEach(e -> {
            List<String> resourceIdList = e.getResourceList();
            if (!CollectionUtils.isEmpty(resourceIdList)) {
                idList.addAll(resourceIdList);
            }
        });

        Map<String, ResourceVo> resourceVoMap = getResourceMap(idList);

        for (WareVo wareVo : wareVoList) {
            List<String> resourceIdList = wareVo.getResourceList();
            if (CollectionUtils.isEmpty(resourceIdList)) break;

            List<ResourceVo> resourceVoList = new ArrayList<>();
            for (String resourceId : resourceIdList) {
                ResourceVo resourceVo = resourceVoMap.get(resourceId);
                if (resourceVo != null) {
                    resourceVoList.add(resourceVo);
                }
            }
            wareVo.setResourceVoList(resourceVoList);
        }
    }

    public void setWareUrl(WareVo wareVo) {
        if (wareVo == null || wareVo.getResourceList() == null) {
            return;
        }

        List<String> resourceIdList = wareVo.getResourceList();
        if (CollectionUtils.isEmpty(resourceIdList)) {
            return;
        }

        Map<String, ResourceVo> resourceVoMap = getResourceMap(resourceIdList);

        List<ResourceVo> resourceVoList = new ArrayList<>();
        for (String resourceId : resourceIdList) {
            ResourceVo resourceVo = resourceVoMap.get(resourceId);
//            ResourceVo resourceVo = (ResourceVo) resourceVoMap.values().toArray()[0];
            if (resourceVo != null) {
                resourceVoList.add(resourceVo);
            }
        }
        wareVo.setResourceVoList(resourceVoList);
    }
}
