package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.dto.CourseDTO;
import cn.rankin.data.api.product.dto.PackageDTO;
import cn.rankin.data.api.product.dto.ProductSearchDTO;
import cn.rankin.data.api.product.dto.SupportDTO;
import cn.rankin.data.api.product.entity.Course;
import cn.rankin.data.api.product.entity.Package;
import cn.rankin.data.api.product.entity.Product;
import cn.rankin.data.api.product.entity.Support;
import cn.rankin.productservice.service.ProductService;
import com.sun.xml.internal.bind.v2.runtime.reflect.Lister;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "/product")
public class ProductController {

    @Autowired
    private ProductService productService;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<Page<Product>> search(ProductSearchDTO searchDTO) {
        Product product = new Product();
        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            product.setCode(String.format("%" + code + "%"));
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            product.setName(String.format("%" + name + "%"));
        }

        product.setType(searchDTO.getType());

        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        APIResult<Page<Product>> pageAPIResult = productService.search(product, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);

        return pageAPIResult;
    }

    @RequestMapping(value = "/{productId}", method = RequestMethod.GET)
    public APIResult getProduct(@PathVariable("productId") String productId) {
        return productService.get(productId);
    }

    @RequestMapping(value = "/course", method = RequestMethod.PUT)
    public APIResult updateCourse(@RequestBody CourseDTO courseDTO) {
        return productService.update(courseDTO);
    }

    @RequestMapping(value = "/support", method = RequestMethod.PUT)
    public APIResult<Support> updateSupport(@RequestBody SupportDTO supportDTO) {
        return productService.update(supportDTO);
    }

    @RequestMapping(value = "/package", method = RequestMethod.PUT)
    public APIResult<Package> updatePackage(@RequestBody PackageDTO packageDTO) {
        return productService.update(packageDTO);
    }

    @RequestMapping(value = "/course", method = RequestMethod.POST)
    public APIResult<Course> createCourse(@RequestBody CourseDTO courseDTO) {
        return productService.create(courseDTO);
    }

    @RequestMapping(value = "/support", method = RequestMethod.POST)
    public APIResult<Support> createSupport(@RequestBody SupportDTO supportDTO) {
        return productService.create(supportDTO);
    }

    @RequestMapping(value = "/package", method = RequestMethod.POST)
    public APIResult<Package> createPackage(@RequestBody PackageDTO packageDTO) {
        return productService.create(packageDTO);
    }

    @RequestMapping(value = "/package/{productId}", method = RequestMethod.GET)
    public APIResult<Package> getPackage(@PathVariable("productId") String productId) {
        return productService.getPackage(productId);
    }

    @RequestMapping(value = "/{productId}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("productId") String productId) {
        return productService.delete(productId);
    }

    @RequestMapping(value = "/ids", method = RequestMethod.GET)
    public APIResult<List<Product>> findByPids(@RequestParam("id") List<String> pidList) {
        return productService.findByPids(pidList);
    }
}
