package cn.rankin.task.service;

import cn.rankin.common.utils.enums.OrderDetailStatusEnum;
import cn.rankin.data.api.trade.entity.Order;
import cn.rankin.data.api.trade.entity.OrderDetail;
import cn.rankin.data.api.trade.entity.OrderGoods;
import cn.rankin.task.repository.OrderDetailRepository;
import cn.rankin.task.repository.OrderGoodsRepository;
import cn.rankin.task.repository.OrderRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class OrderService {

    @Autowired
    private OrderRepository orderRepository;

    @Autowired
    private OrderDetailRepository orderDetailRepository;

    @Autowired
    private OrderGoodsRepository orderGoodsRepository;

    public List<Order> findSplitList(Integer limit) {
        List<Order> orderList = orderRepository.findSplitOrders(limit);
        if (CollectionUtils.isEmpty(orderList)) {
            return orderList;
        }

        List<String> orderIdList = new ArrayList<>();
        for (Order order : orderList) {
            orderIdList.add(order.getId());
        }

        List<OrderGoods> orderGoodsList = orderGoodsRepository.findByOrderIds(orderIdList);
        Map<String, List<OrderGoods>> orderGoodsMap = new HashMap<>();
        for (OrderGoods orderGoods : orderGoodsList) {
            String orderId = orderGoods.getOrderId();
            if (!orderGoodsMap.containsKey(orderId)) {
                List<OrderGoods> tmpList = new ArrayList<>();
                tmpList.add(orderGoods);
                orderGoodsMap.put(orderId, tmpList);
            }else {
                List<OrderGoods> tmpList = orderGoodsMap.get(orderId);
                tmpList.add(orderGoods);
            }
        }

        for (Order order : orderList) {
            String orderId = order.getId();
            List<OrderGoods> tmpList = orderGoodsMap.get(orderId);
            order.setGoods(tmpList);
        }

        return orderList;
    }

    @Transactional
    public Boolean split(String orderId, List<OrderDetail> orderDetailList) {
        Integer count = 0;
        if (orderDetailList.size() >= 2) {
            count = orderRepository.split(orderId);
        }else {
            count = orderRepository.process(orderId);
        }

        if (count == 0) {
            log.error("更新原订单分拆状态失败, orderId={}", orderId);
            return Boolean.FALSE;
        }

        for (OrderDetail orderDetail : orderDetailList) {
            orderDetail.setOrderStatus(OrderDetailStatusEnum.FORSEND);
            OrderDetail one = orderDetailRepository.save(orderDetail);
            String detailId = one.getId();
            List<String> goodsIdList = new ArrayList<>();
            List<OrderGoods> orderGoodsList = orderDetail.getGoods();
            for (OrderGoods orderGoods : orderGoodsList) {
                goodsIdList.add(orderGoods.getGoodsId());
            }
            Integer rowcount = orderGoodsRepository.setDetailId(detailId, orderId, goodsIdList);
            if (rowcount > 0) {
                log.info("orderId={}, 已成功分拆一单, detailId={}", orderId, detailId);
            }
        }
        return Boolean.TRUE;
    }
}

